/**
 * Dragon - SOA Governance Platform.
 * Copyright (c) 2008 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * EndpointManagerImpl.java
 * -------------------------------------------------------------------------
 */

package org.ow2.dragon.service.deployment;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.ow2.dragon.aop.annotation.CheckAllArgumentsNotNull;
import org.ow2.dragon.aop.annotation.CheckArgumentsNotNull;
import org.ow2.dragon.api.service.deployment.DeploymentException;
import org.ow2.dragon.api.service.deployment.EndpointManager;
import org.ow2.dragon.api.to.RequestOptionsTO;
import org.ow2.dragon.api.to.common.KeyedRefTO;
import org.ow2.dragon.api.to.deployment.CommentTO;
import org.ow2.dragon.api.to.deployment.EndpointTO;
import org.ow2.dragon.persistence.bo.common.Category;
import org.ow2.dragon.persistence.bo.common.CategoryBag;
import org.ow2.dragon.persistence.bo.common.CategoryValue;
import org.ow2.dragon.persistence.bo.common.Comment;
import org.ow2.dragon.persistence.bo.common.KeyedReference;
import org.ow2.dragon.persistence.bo.common.Rating;
import org.ow2.dragon.persistence.bo.deployment.Endpoint;
import org.ow2.dragon.persistence.dao.RequestOptions;
import org.ow2.dragon.persistence.dao.UniversalUnifiedDAO;
import org.ow2.dragon.persistence.dao.deployment.CommentDAO;
import org.ow2.dragon.persistence.dao.deployment.EndpointDAO;
import org.ow2.dragon.service.TransfertObjectAssembler;

/**
 * @author ofabre - eBM WebSourcing
 * 
 */
public class EndpointManagerImpl implements EndpointManager {

    private EndpointDAO endpointDAO;

    private UniversalUnifiedDAO universalUnifiedDAO;

    private TransfertObjectAssembler transfertObjectAssembler;

    // Added By TIAR Abderrahmane
    private CommentDAO commentDAO;

    public void setCommentDAO(final CommentDAO commentDAO) {
        this.commentDAO = commentDAO;
    }

    // End

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.dragon.ui.businessdelegate.deployment.EndpointManager#createEndpoint
     * (org.ow2.dragon.ui.model.to.deployment.EndpointTO)
     */
    public String createEndpoint(final EndpointTO endpointTO) throws DeploymentException {
        throw new RuntimeException("not implemented method");
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.dragon.ui.businessdelegate.deployment.EndpointManager#getAllEndpoints
     * (org.ow2.dragon.ui.model.to.RequestOptions)
     */
    public List<EndpointTO> getAllEndpoints(final RequestOptionsTO requestOptionsTO)
            throws DeploymentException {
        final List<EndpointTO> result = new ArrayList<EndpointTO>();
        final List<Endpoint> endpoints = this.endpointDAO.getAll(this.transfertObjectAssembler
                .toEndpointRequestOptions(requestOptionsTO));
        this.toEndpointsTO(result, endpoints);
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.dragon.ui.businessdelegate.deployment.EndpointManager#getEndpoint
     * (java.lang.String)
     */
    @CheckAllArgumentsNotNull
    public EndpointTO getEndpoint(final String endpointId, final String user)
            throws DeploymentException {
        // retrieve person bo
        final Endpoint endpointBO = this.endpointDAO.get(endpointId);
        // create person to from bo
        final EndpointTO endpointTO = this.toEndpointTO(endpointBO, user);
        return endpointTO;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.dragon.ui.businessdelegate.deployment.EndpointManager#
     * getEndpointsByBinding(java.lang.String,
     * org.ow2.dragon.ui.model.to.RequestOptions)
     */
    public List<EndpointTO> getEndpointsByBinding(final String bindingId,
            final RequestOptionsTO requestOptionsTO) throws DeploymentException {
        throw new RuntimeException("not implemented method");
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.dragon.ui.businessdelegate.deployment.EndpointManager#
     * getEndpointsByTechService(java.lang.String,
     * org.ow2.dragon.ui.model.to.RequestOptions)
     */
    public List<EndpointTO> getEndpointsByTechService(final String techServiceId,
            final RequestOptionsTO requestOptionsTO) throws DeploymentException {
        final List<EndpointTO> result = new ArrayList<EndpointTO>();

        // Search endpoints
        RequestOptions requestOptions = this.transfertObjectAssembler
                .toEndpointRequestOptions(requestOptionsTO);
        if (requestOptions != null) {
            requestOptions.setCaseSensitive(true);
        }
        final List<Endpoint> endpoints = this.endpointDAO.searchEquals(
                new String[] { techServiceId }, new String[] { "technicalService.id" },
                requestOptions);
        this.toEndpointsTO(result, endpoints);

        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.dragon.ui.businessdelegate.deployment.EndpointManager#removeEndpoint
     * (java.lang.String)
     */
    public void removeEndpoint(final String endpointId) throws DeploymentException {
        this.endpointDAO.remove(endpointId);
    }

    public void setEndpointDAO(final EndpointDAO endpointDAO) {
        this.endpointDAO = endpointDAO;
    }

    public void setTransfertObjectAssembler(final TransfertObjectAssembler transfertObjectAssembler) {
        this.transfertObjectAssembler = transfertObjectAssembler;
    }

    private void toEndpointsTO(final List<EndpointTO> result, final Collection<Endpoint> endpoints) {
        if ((endpoints != null) && !endpoints.isEmpty()) {
            for (final Endpoint endpoint : endpoints) {
                EndpointTO endpointTO = toEndpointTO(endpoint, null);
                endpointTO.setGlobalRating(updateGlobalRating(endpointTO.getId()));
                result.add(endpointTO);
            }
        }
    }

    private EndpointTO toEndpointTO(final Endpoint endpoint, final String user) {
        return this.transfertObjectAssembler.toEndpointTO(endpoint, user);
    }

    public String updateEndpoint(final EndpointTO endpointTO) throws DeploymentException {
        // Retrieve endpoint
        final Endpoint endpoint = this.endpointDAO.get(endpointTO.getId());
        if (endpoint == null) {
            throw new DeploymentException(
                    "Your are trying to update a non existing Endpoint with id: "
                            + endpointTO.getId());
        }

        // update fields
        this.transfertObjectAssembler.toEndpointBO(endpointTO, endpoint);
        this.endpointDAO.save(endpoint);
        return endpoint.getId();
    }

    public float updateGlobalRating(String endpointId) {
        Endpoint endpoint = endpointDAO.get(endpointId);
        float globalRating = 0f;
        if (!endpoint.getRatings().isEmpty()) {
            Iterator<Rating> iter = endpoint.getRatings().iterator();
            int size = endpoint.getRatings().size();
            while (iter.hasNext()) {
                globalRating = globalRating + iter.next().getMyRating();
            }
            globalRating = globalRating / size;
        }
        return globalRating;
    }

    public void updateComments(String endpointId, CommentTO comment) {
        Endpoint endpoint = endpointDAO.get(endpointId);

        if (endpoint != null) {
            final Comment commentTemp = new Comment();

            commentTemp.setContent(comment.getContent());
            commentTemp.setDate(comment.getDate());
            commentTemp.setUserName(comment.getUserName());
            commentTemp.setId(comment.getId());

            endpoint.addComment(commentTemp);
        }
    }

    public List<EndpointTO> getEndpointsByTag(String tag) {
        List<EndpointTO> result = new ArrayList<EndpointTO>();
        final List<Endpoint> allEndpoints = this.endpointDAO.getAll();
        final List<Endpoint> endpoints = new ArrayList<Endpoint>();

        Iterator<Endpoint> iter = allEndpoints.iterator();

        while (iter.hasNext()) {
            final Endpoint endpointTemp = iter.next();
            if (endpointTemp.isTagged(tag)) {
                endpoints.add(endpointTemp);
            }
        }
        toEndpointsTO(result, endpoints);
        return result;
    }

    public void removeTag(String tag, String endpointId) {
        Endpoint endpoint = this.endpointDAO.get(endpointId);
        Set<String> tags = endpoint.getTags();
        tags.remove(tag);
        endpoint.setTags(tags);
    }

    public void removeComment(String endpointId, String commentId) {
        Endpoint endpoint = this.endpointDAO.get(endpointId);
        Comment comment = this.commentDAO.get(commentId);
        List<Comment> comments = endpoint.getComments();

        comments.remove(comment);
        endpoint.setComments(comments);
    }

    public List<KeyedRefTO> getCategoriesForEndpoint(String endpointId) throws DeploymentException {
        Endpoint endpoint = this.endpointDAO.get(endpointId);
        if (endpoint == null) {
            throw new DeploymentException("No endpoint found for the given id: " + endpointId);
        }

        return transfertObjectAssembler.toCategoriesTO(endpoint.getCategoryBag());
    }

    public void removeCategories(String techServiceId, List<String> categoryIds) {
        if (categoryIds != null) {
            // Retrieve endpoint
            Endpoint endpoint = endpointDAO.get(techServiceId);

            // Unlink all categories
            List<KeyedReference> keyRefs = universalUnifiedDAO.getAll(KeyedReference.class,
                    categoryIds);
            endpoint.getCategoryBag().getKeyedReferences().removeAll(keyRefs);

            // Delete categories
            universalUnifiedDAO.removeAll(keyRefs);
        }
    }

    @CheckAllArgumentsNotNull
    public void addCategory(String techServiceId, String categoryId, String categoryValueId)
            throws DeploymentException {
        // Retrieve endpoint
        Endpoint endpoint = endpointDAO.get(techServiceId);

        // Retrieve category
        Category category = (Category) universalUnifiedDAO.get(Category.class, categoryId);

        // Retrieve category value
        CategoryValue value = (CategoryValue) universalUnifiedDAO.get(CategoryValue.class,
                categoryValueId);

        if (endpoint != null && category != null) {
            // add category to service
            KeyedReference keyedReference = new KeyedReference();
            keyedReference.setTmodel(category);
            keyedReference.setKeyName(value.getDescription());
            keyedReference.setKeyValue(value.getValue());
            this.validateCategory(endpoint, keyedReference);

            CategoryBag categoryBag = endpoint.getCategoryBag();
            if (categoryBag != null) {
                categoryBag.addKeyedReference(keyedReference);
            } else {
                categoryBag = new CategoryBag();
                categoryBag.addKeyedReference(keyedReference);
                endpoint.setCategoryBag(categoryBag);
            }
        } else {
            throw new DeploymentException(
                    "You have specified unknown endpoint, category system or category value");
        }

        endpointDAO.save(endpoint);
    }

    private void validateCategory(Endpoint endpoint, KeyedReference keyedReference)
            throws DeploymentException {
        CategoryBag categoryBag = endpoint.getCategoryBag();
        if (categoryBag != null) {
            List<KeyedReference> keyedReferences = categoryBag.getKeyedReferences();
            if (keyedReferences != null) {
                if (keyedReferences.contains(keyedReference)) {
                    throw new DeploymentException("Category already added to this endpoint.");
                }
            }
        }
    }

    @CheckArgumentsNotNull
    public void addCategory(String techServiceId, String categoryId, String categoryValue,
            String categoryDesc) throws DeploymentException {
        // Retrieve endpoint
        Endpoint endpoint = endpointDAO.get(techServiceId);

        // Retrieve category
        Category category = (Category) universalUnifiedDAO.get(Category.class, categoryId);

        if (endpoint != null && category != null) {
            // add category to service
            KeyedReference keyedReference = new KeyedReference();
            keyedReference.setTmodel(category);
            keyedReference.setKeyName(categoryDesc);
            keyedReference.setKeyValue(categoryValue);
            this.validateCategory(endpoint, keyedReference);

            CategoryBag categoryBag = endpoint.getCategoryBag();
            if (categoryBag != null) {
                categoryBag.addKeyedReference(keyedReference);
            } else {
                categoryBag = new CategoryBag();
                categoryBag.addKeyedReference(keyedReference);
                endpoint.setCategoryBag(categoryBag);
            }
        } else {
            throw new DeploymentException(
                    "You have specified unknown endpoint, category system or category value");
        }

        endpointDAO.save(endpoint);
    }

    public void setUniversalUnifiedDAO(UniversalUnifiedDAO universalUnifiedDAO) {
        this.universalUnifiedDAO = universalUnifiedDAO;
    }
}
